/*ATTRIB.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#include <string.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "stack.h"
#include "list.h"
#include "exec.h"
#include "gene.h"
#include "nodetype.h"
#include "tab.h"
#include "memory.h"
#include "macros.h"

#include "io.h"
#include "vars.h"

#define DESC_ATTR(t,x,b) do { \
                           if (HAS_SOME_ATTR(t, x##_ATTR)) { \
                             strcat(b, #x " "); \
                           } else { \
                             int i; \
                             for (i = 0; i < strlen(#x); i++) { \
                               strcat(b, "-"); \
                             } \
                             strcat(b, " "); \
                           } \
                         } while (0)

char *attrib_name(int t)
{
  static char buffer[128];
  buffer[0] = '\0';

  DESC_ATTR(t, INT, buffer);
  DESC_ATTR(t, OBJ, buffer);

  DESC_ATTR(t, SING, buffer);
  DESC_ATTR(t, LIST, buffer);

  DESC_ATTR(t, RET, buffer);
  DESC_ATTR(t, GENE, buffer);
  DESC_ATTR(t, LDF, buffer);
  DESC_ATTR(t, VAR, buffer);

  DESC_ATTR(t, VAL, buffer);
  DESC_ATTR(t, PNT, buffer);

  strcat(buffer, "( ");

  DESC_ATTR(t, INS, buffer);
  DESC_ATTR(t, DEL, buffer);
  DESC_ATTR(t, END, buffer);

  strcat(buffer, ": ");

  DESC_ATTR(t, ERROR, buffer);

  strcat(buffer, ")");
  return buffer;
}

/* Does a full comparison of two Attributes, including
** the parameter types for a gene.  Note that this function is
** asymmetric, the question asked is:
**		"Is "b" *a* valid answer to a request for type(s) "a" ?  */

int Attrib_Compare(AL_Attrib a, AL_Attrib b) {
  int i;
  if (!ACCEPTS_ATTR(a[0], b[0])) {
    return 0;			/* base type incompatible */
  }
/* Is "b" a gene ?  We mustn't check if "a" is because "a" might also
** accept things that are not.  */

  if (ACCEPTS_ATTR(b[0], GENE_ATTR)) {
    /* if "b" *is* a gene then check that "a" can
    ** supply the correct argument types */
    for (i = 2; i < b[1] + 2; i++) {
      /* Next line is the other way around !  Its the params of "b"
      ** that must find the types offered by "a" acceptible */
      if (!ACCEPTS_ATTR(b[i], a[i])) {
	return 0;
      }
    }
  }
  return 1;
}

int type_from_name(char *name) {
  int ret = 0;

  printf("parsing name \"%s\".\n", name);

  if (*name == '*') {
    ADD_ATTR(ret, PNT_ATTR);
    name++;
  } else {
    ADD_ATTR(ret, VAL_ATTR);
  }

  if (*name == 'I' || *name == 'i') {
    ADD_ATTR(ret, INT_ATTR);
  } else if (*name == 'O' || *name == 'o') {
    ADD_ATTR(ret, OBJ_ATTR);
  }

  if (*name == 'I' || *name == 'O') {
    ADD_ATTR(ret, LIST_ATTR);
    name++;
  } else if (*name == 'i' || *name == 'o') {
    ADD_ATTR(ret, SING_ATTR);
    name++;
  } else {
    TRACEBACK("warning: TYPE/MULTIPLICITY missing in name, assuming SINGLE/INT");
    ADD_ATTR(ret, SING_ATTR | INT_ATTR);
  }

  if (*name == 'R' || *name == 'r') {
    ADD_ATTR(ret, RET_ATTR);
    name++;
  } else if (*name == 'G' || *name == 'g') {
    ADD_ATTR(ret, GENE_ATTR);
    name++;
  } else if (*name == 'L' || *name == 'l') {
    ADD_ATTR(ret, LDF_ATTR);
    name++;
  } else if (*name == 'V' || *name == 'v') {
    ADD_ATTR(ret, VAR_ATTR);
    name++;
  } else {
    /* default FLAVOUR for a name should be VAR, we don't wan't to have to
    ** put a gratuitous 'V' in all VAR and PARAM names so remove this warning
    ** after debugging. */
    TRACEBACK("warning: FLAVOUR missing in name, assuming VAR");
    ADD_ATTR(ret, VAR_ATTR);
  }

  if (ACCEPTS_ATTR(RET_ATTR | PNT_ATTR, ret) ||
      ACCEPTS_ATTR(VAR_ATTR | VAL_ATTR, ret)) {
    TRACEBACK("warning: meaningless FLAVOUR/REFERENCE combination in name.");
  }

  if (HAS_NONE_ATTR(TYPE_ATTR, ret)) {
    TRACEBACK("warning: no type in name");
  }
  if (HAS_NONE_ATTR(MULT_ATTR, ret)) {
    TRACEBACK("warning: no multiplicity in name");
  }
  if (HAS_NONE_ATTR(FLAV_ATTR, ret)) {
    TRACEBACK("warning: no flavour in name");
  }
  if (HAS_NONE_ATTR(REF_ATTR, ret)) {
    TRACEBACK("warning: no reference-status in name");
  }

  return ret;
}
