/*CTRL_N.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "stack.h"
#include "list.h"
#include "exec.h"
#include "nodetype.h"
#include "tab.h"
#include "macros.h"

/* #include "vars.h" */

/******************************* IF *******************************
** IF; Evaluates first child, then evaluates and returns
** one of its other two subtreees depending on whether the first
** returned zero or non-zero
*/

void IF_exe() {
  switch (STATUS++) {
      case 0:
      Enter_Node(CHILD(THISNODE, 0));
      break;

    case 1:
      if (IARG(0)) {
        Enter_Node(CHILD(THISNODE, 1));
      } else {
        Enter_Node(CHILD(THISNODE, 2));
      }
      break;

    case 2:
      IRETVAL = IPOP();
      Exit_Node();
      break;
  }
}

void IF_attr(struct AL_Node * where, AL_Attrib buffer) {
  DECL_ATTRIB(temp);
  ATTR(CHILD(where, 1), buffer);
  ATTR(CHILD(where, 0), temp);
  MIX_FLAGS(buffer, temp);
  ATTR(CHILD(where, 2), temp);
  MIX_FLAGS(buffer, temp);
}

/* could check that ret_type when answering for child#2 matches that
** for child#1 but this shouldn't happen ? <IGB> */

int IF_res(AL_Attrib ret_type,
           struct AL_Node * where,
           int i,
           AL_Attrib buffer) {
  switch (i) {
    case 0:
      buffer[0] = VAL_ATTR | INT_ATTR | SING_ATTR | RET_ATTR;
      break;
    case 1: case 2:
      COPY_ATTR(buffer, ret_type);
      ADD_ATTR(buffer[0], INS_ATTR | DEL_ATTR);
      break;
    case 3:
      buffer[0] = END_ATTR;
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
  return 1;
}


/******************************* ALL *******************************
** ALL; Evaluates children in order.  Returns value of first child.
** This function is also used as a placeholder for the ExeFun of the
** PARENTS node type.
**
** Hmmm, wouldn't a NULL be better ? <IGB>
*/

void ALL_exe() {
  if (STATUS == 1) {
    VRETVAL = VPOP();
  } else if (STATUS > 1) {
    SP--;	/* discard unwanted return value */
  }

  if (STATUS < NUMBER(THISNODE)) {
    Enter_Node(CHILD(THISNODE, STATUS++));
    return;
  }
  Exit_Node();
}

int ALL_res(AL_Attrib ret_type,
                  struct AL_Node * where,
                  int i,
                  AL_Attrib buffer) {
  if (i == 0) {
    COPY_ATTR(buffer, ret_type);
    ADD_ATTR(buffer[0], INS_ATTR | DEL_ATTR);
    /* We can delete/insert child#0 but when we do, the next child
    ** will be forced to be of the approppriate type.  We can't
    ** end before the first child as this leaves no children and
    ** we wouldn't have anything to supply a return val of the correct
    ** type. */
  } else {
    buffer[0] = ANY | END_ATTR | INS_ATTR | DEL_ATTR;
  }
  return 1;
}

void ALL_attr(struct AL_Node * where, AL_Attrib buffer) {
  int i;
  DECL_ATTRIB(temp);
  ATTR(CHILD(where, NUMBER(where) - 1), buffer);
  for(i = 0; i < NUMBER(where) - 1; i++) {
    ATTR(CHILD(where, i), temp);
    MIX_FLAGS(buffer, temp);
  }
}

/******************************* REP *******************************
** REP; Evaluates second-child the number of times returned by the
** first.  Returns value of last such call.
*/

void REP_exe() {
  if (STATUS++ == 0) {
    Enter_Node(CHILD(THISNODE, 0));
    return;
  }
  if (IARG(0)-- > 0) {
    if (STATUS != 2) {                /* wierd but correct, honest */
      POP();                        /* don't accumulate multiple returns on stack */
    }
    Enter_Node(CHILD(THISNODE, 1));
    return;
  }
  VRETVAL = VPOP();

  Exit_Node();
}

void REP_attr(struct AL_Node * where, AL_Attrib buffer) {
  DECL_ATTRIB(temp);
  ATTR(CHILD(where, 1), buffer);
  ATTR(CHILD(where, 0), temp);
  MIX_FLAGS(buffer, temp);
}

int REP_res(AL_Attrib ret_type,
            struct AL_Node * where,
            int i,
            AL_Attrib buffer) {
  switch (i) {
    case 0:
      buffer[0] = VAL_ATTR | INT_ATTR | SING_ATTR | RET_ATTR;
      break;
    case 1:
      COPY_ATTR(buffer, ret_type);
      ADD_ATTR(buffer[0], DEL_ATTR | INS_ATTR);
      break;
    case 2:
      buffer[0] = END_ATTR;
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
  return 1;
}
