/*LDF.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "exec.h"
#include "nodetype.h"
#include "tab.h"
#include "vars.h"
#include "macros.h"

/******************************* DEC *******************************
** LDF declaration
*/

void DEC_exe() {
  if (!STATUS++) {
    PPUSH(LDFP);
    LDFP = FP;

    Enter_Node(CHILD(THISNODE, 2));
  } else {
    VRETVAL = VPOP();
    LDFP = PPOP();
    Exit_Node();
  }
}

Enum AL_Attrib DEC_attr(struct AL_Node * where) {
  /* In this function we report to our caller that we can supply every
  ** data type that there is! This is a lie, but see the next function.
  */

  Enum AL_Attrib f = ATTR(CHILD(where, 2));
  return f | I_ATTR | O_ATTR | L_I_ATTR | L_O_ATTR | VAL_ATTR;
}
/* This works exactly like the GEN version, except with customised
** handlers for two of the three children. The first we know is a
** PRIV_ATTR name, so we do not even consider crossover. The
** second is handled exactly like the default. The third is the most
** interesting.
** We can compare was_where and sister to detect if this node has
** been copied into the same place in the tree of whether it is a
** crossover mutation. If it has been copied then we proceed identically
** to the normal handler.
** If it has been created through crossover then we are in the situation
** where the mutation engine has called our bluff. Previously this node
** lied about what data types it supported, and now it has to deliver itself
** in that form. Somehow we have to find a third child that can provide us
** with a value of the correct type, without even being told directly what
** that type is.
** Fortunately we can be cunning. The sister node previously occupied the
** site at which this node is to be placed. We can either interrogate our
** sister to determine her type, or clone her directly.
*/

int DEC_copy(struct AL_Node * where, struct AL_Node * was_where,
             struct AL_Node * node_pool,
             struct AL_Node * sister) {
  NUMBER(where) = 3;
  where->private = malloc((size_t) (sizeof(struct AL_Node *) * 3));
  if (!where->private)
    return 0;

  /* Copy the LDF name */
  CHILD(where, 0) = CM_AL_Node(CHILD(was_where, 0), node_pool, CHILD(was_where, 0));
  if (!CHILD(where, 0)) {
    free(where->private);
    return 0;
  }
  /* Copy the LDF body */
  CHILD(where, 1) = CM_AL_Node(CHILD(was_where, 1), node_pool, CHILD(was_where, 1));
  if (!CHILD(where, 1)) {
    DEL(CHILD(where, 0));
    free(where->private);
    return 0;
  }
  if (was_where == sister) {
    /* Copy the original subtree */
    CHILD(where, 2) = CM_AL_Node(CHILD(was_where, 2), node_pool, CHILD(was_where, 2));
  } else {
    /* Copy the sister subtree */
    CHILD(where, 2) = CM_AL_Node(sister, node_pool, sister);
  }

  if (!CHILD(where, 2)) {
    DEL(CHILD(where, 0));
    DEL(CHILD(where, 1));
    free(where->private);
    return 0;
  }
  return 1;
}



/******************************* LDF *******************************
** LDF call
*/


void LDF_exe() {
  union AL_Stack_Value *searching;
  struct AL_Node *possible_ldf;
  if (!STATUS++) {
    /* searching is an old FP value; searching[0].p
    ** points to the LDF AL_Node. searching[2] is its local
    ** variable; a pointer back down the list of other stack frames.
    */
    searching = LDFP;

    while (searching) {
      possible_ldf = (struct AL_Node *) searching[0].p;

      if (NUMBER(CHILD(possible_ldf, 0)) == NUMBER(CHILD(THISNODE, 0))) {
        Enter_Node(CHILD(possible_ldf, 1));
        return;
      }
      searching = (union AL_Stack_Value *) (searching[2].p);
    }

    /* should never get here */

    printf("LDF not found\r\n");
    abort();
  } else {
    VRETVAL = VPOP();
    Exit_Node();
  }
}

Enum AL_Attrib LDF_attr(struct AL_Node * where) {
  return VAL_ATTR | name_to_attrib(NUMBER(CHILD(where, 0)));
}

Enum AL_Attrib LDF_res(struct AL_Node * where, int i) {
  switch (i) {
      case 0:return PRIV_ATTR | name_to_attrib(NUMBER(CHILD(where, 0)));
    default:
      return IMPOS_ATTR;
  }
}
