/*NODETYPE.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

/*#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <ctype.h>

#include "stack.h"
#include "list.h"
#include "exec.h"
#include "gene.h"
#include "tab.h"
#include "memory.h"
#include "macros.h"

#include "io.h"
#include "vars.h"
*/

#include <stdio.h>

#include "cnstants.h"
#include "attrib.h"
#include "preamble.h"
#include "nodetype.h"

const struct AL_NodeType PARENT_TYPE =
{NULL_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 PARENT_attr,
 NULL_res,
 "PARENTS"};
const struct AL_NodeType GENE_TYPE =
{NULL_exe,
 GENE_ser,
 GENE_dser,
 GENE_iter,
 GENE_del,
 GENE_copy,
 GENE_attr,
 GENE_res,
 "GENE"};
const struct AL_NodeType GCALL_TYPE =
{GCALL_exe,
 GCALL_ser,
 NULL_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 GCALL_attr,
 GCALL_res,
 "GCALL"};
const struct AL_NodeType GN_TYPE =
{GN_exe,
 GN_ser,
 NULL_dser,
 GN_iter,
 GN_del,
 GN_copy,
 GN_attr,
 LEAF_res,
 " GN"};
const struct AL_NodeType K_TYPE =
{K_exe,
 K_ser,
 NULL_dser,
 K_iter,
 K_del,
 K_copy,
 K_attr,
 LEAF_res,
 "K"};
const struct AL_NodeType VN_TYPE =
{VN_exe,
 VN_ser,
 NULL_dser,
 VN_iter,
 VN_del,
 VN_copy,
 VN_attr,
 LEAF_res,
 " VN"};
const struct AL_NodeType PN_TYPE =
{PN_exe,
 PN_ser,
 NULL_dser,
 PN_iter,
 PN_del,
 PN_copy,
 PN_attr,
 LEAF_res,
 " PN"};
const struct AL_NodeType SET_TYPE =
{SET_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 SET_attr,
 SET_res,
 "SET"};
const struct AL_NodeType GET_TYPE =
{GET_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 GET_attr,
 GET_res,
 "GET"};
const struct AL_NodeType ALL_TYPE =
{ALL_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 ALL_attr,
 ALL_res,
 "ALL"};
const struct AL_NodeType REP_TYPE =
{REP_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 REP_attr,
 REP_res,
 "REP"};
const struct AL_NodeType PRI_TYPE =
{PRI_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 PRI_res,
 "PRINT"};
const struct AL_NodeType LPRI_TYPE =
{LPRI_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 LI_attr,
 LPRI_res,
 "LPRINT"};
const struct AL_NodeType IF_TYPE =
{IF_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 IF_attr,
 IF_res,
 "IF"};
const struct AL_NodeType SER_TYPE =
{SER_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 SER_attr,
 SER_res,
 "SER"};
#if 0
const struct AL_NodeType LDF_TYPE =
{LDF_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 LDF_attr,
 LDF_res,
 "LDF"};
const struct AL_NodeType DEC_TYPE =
{DEC_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 DEC_copy,
 DEC_attr,
 NULL,
 "DEC"};
#endif
const struct AL_NodeType DIV_TYPE =
{DIV_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 II_res,
 "/"};
const struct AL_NodeType SUBT_TYPE =
{SUBT_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 II_res,
 "-"};
const struct AL_NodeType MULT_TYPE =
{MULT_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 MI_res,
 "*"};
const struct AL_NodeType PLUS_TYPE =
{PLUS_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 MI_res,
 "+"};
const struct AL_NodeType OR_TYPE =
{OR_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 MI_res,
 "||"};
const struct AL_NodeType AND_TYPE =
{AND_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 MI_res,
 "&&"};
const struct AL_NodeType XOR_TYPE =
{XOR_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 I_attr,
 MI_res,
 "^"};
const struct AL_NodeType EMPTY_TYPE =
{EMPTY_exe,
 GEN_ser,
 GEN_dser,
 GEN_iter,
 GEN_del,
 GEN_copy,
 EMPTY_attr,
 LEAF_res,
 "EMPTY"};

const struct AL_NodeType *AL_NodeTypes[] =
{
/* abstract nodes */
  &PARENT_TYPE,
/* gene nodes */
  &GENE_TYPE, &GCALL_TYPE, &GN_TYPE,
/* integer constant */
  &K_TYPE,
/* variable using nodes */
  &VN_TYPE, &PN_TYPE, &GET_TYPE, &SET_TYPE,
/* control nodes */
  &ALL_TYPE, &REP_TYPE, &IF_TYPE,
/* debug nodes */
  &PRI_TYPE, &LPRI_TYPE,
/* list nodes */
  &SER_TYPE, &EMPTY_TYPE,
#if 0
/* LDF nodes */
  &LDF_TYPE, &DEC_TYPE,
#endif
/* arithmetic nodes */
  &DIV_TYPE, &SUBT_TYPE, &MULT_TYPE, &PLUS_TYPE,
/* boolean logic nodes */
  &OR_TYPE, &AND_TYPE, &XOR_TYPE
};

const int NumAL_NodeTypes = (sizeof(AL_NodeTypes) /
                             sizeof(struct AL_NodeType *));
