/*NODETYPE.H************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#ifndef _NODETYPE_H_
#define _NODETYPE_H_

/* This table is equivalent to a C++ virtual methods table;
** one of these tables is created for each type of AL_Node. Instances
** of that AL_Node type contain a pointer to the table.
** In addition to pointers to member functions, this table also
** contains a pointer to the AL_Nodes name        - Toby
**
*/

struct AL_NodeType {
  ExeFun *exe;                /* Call this function to evaluate node */
  SerFun *ser;                /*   ...to save to disk */
  DSerFun *dser;              /*   ...to restore from disk */
  IterFun *iterate;        /*   ...to call a function for each descendant */
  DelFun *del;                /*   ...to delete this and its private */
  CopyFun *copy;              /*   ...copy this node and children */
  AttFun *attrib;             /*   ...the attribs (type) of this node */
  ResFun *restriction;        /*   ...attribs required of children */

  const char name[AL_NAME_MAX + 1];        /* Name of node type */
};


/*
** No matter what file ALang node-functions are defined in, we must
** declare them all here (so always include this file).  All AL_NodeType's
** must also be initialised and stuffed in the array below.
*/

/* Some Error catching versions for use in slots which should
** never be called. */

AttFun NULL_attr;
CopyFun NULL_copy;
DelFun NULL_del;
DSerFun NULL_dser;
ExeFun NULL_exe;
IterFun NULL_iter;
ResFun NULL_res;
SerFun NULL_ser;

/* Real ones */

AttFun ALL_attr;
AttFun DEC_attr;
AttFun EMPTY_attr;
AttFun GCALL_attr;
AttFun GENE_attr;
AttFun GET_attr;
AttFun GN_attr;
AttFun IF_attr;
AttFun I_attr;
AttFun K_attr;
AttFun LDF_attr;
AttFun LI_attr;
AttFun no_attr;
AttFun N_attr;
AttFun O_attr;
AttFun PARENT_attr;
AttFun PN_attr;
AttFun REP_attr;
AttFun SER_attr;
AttFun SET_attr;
AttFun TOP_attr;
AttFun VN_attr;

CopyFun DEC_copy;
CopyFun GENE_copy;
CopyFun GEN_copy;
CopyFun GN_copy;
CopyFun K_copy;
CopyFun PN_copy;
CopyFun VN_copy;

DelFun GENE_del;
DelFun GEN_del;
DelFun GN_del;
DelFun K_del;
DelFun PN_del;
DelFun VN_del;

DSerFun DIV_dser;
DSerFun GENE_dser;
DSerFun GEN_dser;
DSerFun GET_dser;
DSerFun IF_dser;
DSerFun SET_dser;
DSerFun SUBT_dser;
DSerFun TOP_dser;

ExeFun ALL_exe;
ExeFun AND_exe;
ExeFun DEC_exe;
ExeFun DIV_exe;
ExeFun EMPTY_exe;
ExeFun GCALL_exe;
ExeFun GENE_exe;
ExeFun GET_exe;
ExeFun GN_exe;
ExeFun IF_exe;
ExeFun K_exe;
ExeFun LDF_exe;
ExeFun LPRI_exe;
ExeFun MULT_exe;
ExeFun N_exe;
ExeFun OR_exe;
ExeFun PLUS_exe;
ExeFun PRI_exe;
ExeFun PN_exe;
ExeFun REP_exe;
ExeFun SER_exe;
ExeFun SET_exe;
ExeFun SUBT_exe;
ExeFun TOP_exe;
ExeFun VN_exe;
ExeFun XOR_exe;

IterFun GENE_iter;
IterFun GEN_iter;
IterFun GN_iter;
IterFun K_iter;
IterFun PN_iter;
IterFun VN_iter;

ResFun ALL_res;
ResFun EMPTY_res;
ResFun GCALL_res;
ResFun GENE_res;
ResFun GET_res;
ResFun IF_res;
ResFun II_res;
ResFun imposs_res;
ResFun LDF_res;
ResFun LEAF_res;
ResFun LPRI_res;
ResFun MI_res;
ResFun PRI_res;
ResFun REP_res;
ResFun SER_res;
ResFun SET_res;
ResFun TOP_res;

SerFun GCALL_ser;
SerFun GENE_ser;
SerFun GEN_ser;
SerFun GN_ser;
SerFun K_ser;
SerFun PN_ser;
SerFun VN_ser;

extern const struct AL_NodeType PARENT_TYPE;
extern const struct AL_NodeType GENE_TYPE;
extern const struct AL_NodeType GCALL_TYPE;
extern const struct AL_NodeType GN_TYPE;
extern const struct AL_NodeType K_TYPE;
extern const struct AL_NodeType VN_TYPE;
extern const struct AL_NodeType PN_TYPE;
extern const struct AL_NodeType TOP_TYPE;
extern const struct AL_NodeType SET_TYPE;
extern const struct AL_NodeType GET_TYPE;
extern const struct AL_NodeType ALL_TYPE;
extern const struct AL_NodeType REP_TYPE;
extern const struct AL_NodeType PRI_TYPE;
extern const struct AL_NodeType LPRI_TYPE;
extern const struct AL_NodeType IF_TYPE;
extern const struct AL_NodeType SER_TYPE;
#if 0
extern const struct AL_NodeType LDF_TYPE;
extern const struct AL_NodeType DEC_TYPE;
#endif
extern const struct AL_NodeType DIV_TYPE;
extern const struct AL_NodeType SUBT_TYPE;
extern const struct AL_NodeType MULT_TYPE;
extern const struct AL_NodeType PLUS_TYPE;
extern const struct AL_NodeType OR_TYPE;
extern const struct AL_NodeType AND_TYPE;
extern const struct AL_NodeType XOR_TYPE;
extern const struct AL_NodeType EMPTY_TYPE;

extern const struct AL_NodeType *AL_NodeTypes[];
extern const int NumAL_NodeTypes;

#endif /* _NODETYPE_H_ */
