/*VAR_N.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "gene.h"
#include "stack.h"
#include "list.h"
#include "exec.h"
#include "nodetype.h"
#include "tab.h"
#include "memory.h"
#include "macros.h"

#include "vars.h"

/******************************* SET *******************************
** SET; Evaluates first child, assigns the value to its variable and
** also returns it.
*/

void SET_exe() {
  switch (STATUS++) {
    case 0:
      Enter_Node(CHILD(THISNODE, 0));
      break;

    case 1:
      Enter_Node(CHILD(THISNODE, 1));
      break;

    case 2:
      VRETVAL = *(union AL_Stack_Value *)PARG(0) = VARG(1);
      Exit_Node();
  }
}

int SET_res(AL_Attrib ret_type,
            struct AL_Node * where,
            int i,
            AL_Attrib buffer) {
  if (ACCEPTS_ATTR(ret_type[0], VAL_ATTR | RET_ATTR)) {
    return 0;
  }
  switch (i) {
    case 0:
      COPY_ATTR(buffer, ret_type);
      LOSE_ATTR(buffer[0], REF_ATTR | FLAV_ATTR);
      ADD_ATTR(buffer[0], PNT_ATTR | VAR_ATTR);
      break;
    case 1:
      COPY_ATTR(buffer, ret_type);
      LOSE_ATTR(buffer[0], REF_ATTR | FLAV_ATTR);
      ADD_ATTR(buffer[0], VAL_ATTR | RET_ATTR);
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
  return 1;
}

void SET_attr(struct AL_Node * where, AL_Attrib buffer) {
  AL_Attrib temp;
  ATTR(CHILD(where, 1), buffer);	/* same type as child#1 */
  ATTR(CHILD(where, 0), temp);
  MIX_FLAGS(buffer, temp);
}

/******************************* GET *******************************
** GET; returns the value of a variable.
*/

void GET_exe() {
  switch (STATUS++) {
    case 0:
      Enter_Node(CHILD(THISNODE, 0));
      break;

    case 1:
      VRETVAL = *(union AL_Stack_Value *)PARG(0);
      Exit_Node();
  }
}

int GET_res(AL_Attrib ret_type,
            struct AL_Node * where,
            int i,
            AL_Attrib buffer) {
  if (!ACCEPTS_ATTR(ret_type[0], RET_ATTR | VAL_ATTR)) {
    return 0;
  }
  switch (i) {
    case 0:
      COPY_ATTR(buffer, ret_type);
      LOSE_ATTR(buffer[0], REF_ATTR | FLAV_ATTR);
      ADD_ATTR(buffer[0], PNT_ATTR | VAR_ATTR);
      break;
    case 1:
      buffer[0] = END_ATTR;
      break;
    default:
      buffer[0] = ERROR_ATTR;
  }
  return 1;
}

void GET_attr(struct AL_Node * where, AL_Attrib buffer) {
  ATTR(CHILD(where, 0), buffer);
  LOSE_ATTR(buffer[0], REF_ATTR | FLAV_ATTR);
  ADD_ATTR(buffer[0], VAL_ATTR | RET_ATTR);
}

/****************************** VN ********************************
** VN; used internally for names of variables.
**
** The way it works is that the nodes number field refers to a slot in the
** genes local variable details array (Vars) from which the data-type can be
** extracted.  The return value at execution time is a pointer to the
** variables storage which is found by adding the number field to the
** current value of VarPointer.
*/

void VN_exe() {
  PRETVAL = VP + NUMBER(THISNODE);
  Exit_Node();
}

void VN_del(struct AL_Node * where) {
  AL_Node_free(where);
}

/*
**int VN_dser(struct AL_Node * where, FILE * f) {
**
** This will be handled in the main deserialisation routine so that we can
** just use the variable name and not use an explicit node-name.
**
** }
*/

void VN_ser(struct AL_Node * where, FILE * f,
            struct AL_Tab * prev_tab) {
  fputs(CURRENT_GENE->Vars[NUMBER(where)].name, f);
}

void VN_attr(struct AL_Node *where, AL_Attrib buffer) {
  buffer[0] = CURRENT_GENE->Vars[NUMBER(where)].type;
}

int VN_iter(struct AL_Node * where,
            CheckFun * check,
            void *private) {
  return check(where, private);
}

/*
** Just a dummy until I confirm the new procedure.
*/

int VN_copy(struct AL_Node * where,
            struct AL_Node * was_where,
            struct AL_Node * node_pool,
            AL_Attrib ret_type) {
  return 1;
}
