/*GENE_N.C************************************************************* 
** ALang Execution/Mutation Engine: Version 0.6 >> %1
** Part of Project von Nuemann
** Copyright Dr I G Badcoe (I.Badcoe"bristol.ac.uk),
**           Toby Dickenson (htrd@tcp.co.uk) and
**           the Directors of Project von Nuemann.
**
** License is granted for the use of this software only as part of
** Project von Nuemann.
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#include "attrib.h"
#include "gene.h"
#include "stack.h"
#include "list.h"
#include "exec.h"
#include "nodetype.h"
#include "tab.h"
#include "memory.h"
#include "macros.h"

#include "vars.h"
#include "io.h"

void GCALL_exe() {
  if (STATUS == 0) {
    PPUSH(PP);
    PP = SP;
  }

  STATUS++;
  if (STATUS < NUMBER(THISNODE)) {
    Enter_Node(CHILD(THISNODE, STATUS));
  } else if (STATUS == NUMBER(THISNODE)) {
    Enter_Node(CHILD(THISNODE, 0));
  } else if (STATUS == NUMBER(THISNODE) + 1) {
    Enter_Gene(PPOP());
  } else {
    VRETVAL = VPOP();
    Exit_Gene();
    SP = PP;
    PP = PPOP();
    Exit_Node();
  }
}

int GCALL_res(AL_Attrib ret_type,
              struct AL_Node * where,
              int i,
              AL_Attrib buffer) {
  if (!ACCEPTS_ATTR(ret_type[0], RET_ATTR | VAL_ATTR)) {
    return 0;
  }
  if (i == 0) {
    COPY_ATTR(buffer, ret_type);
    LOSE_ATTR(buffer[0], FLAV_ATTR | REF_ATTR);
    ADD_ATTR(buffer[0], GENE_ATTR | PNT_ATTR | LOCAL_ATTR);
  } else if (i <= N2G(CHILD(where, 0))->NumPars) {
    buffer[0] = N2G(CHILD(where, 0))->Type[i + 1];
    /* don't need to alter int's 1-17 of buffer as parameters can't be
    ** genes and so can't have parameters. */
  } else if (i == N2G(CHILD(where, 0))->NumPars + 1) {
    buffer[0] = END_ATTR;
  } else {
    buffer[0] = ERROR_ATTR;
  }
  
  return 1;
}

/* Our return type is a value of the type returned by the gene we call. */

void GCALL_attr(struct AL_Node * where, AL_Attrib buffer) {
  ATTR(CHILD(where, 0), buffer);
  LOSE_ATTR(buffer[0], FLAV_ATTR | REF_ATTR);
  ADD_ATTR(buffer[0], RET_ATTR | VAL_ATTR);
}

void GCALL_ser(struct AL_Node *where, FILE *f, struct AL_Tab *prev) {
  int namelen;
  struct AL_Gene *gene;
  char *genename;
  
  gene = PRIV(CHILD(where, 0));
  genename = gene->Name;
  namelen = strlen(genename) + 1;
  fputs(genename, f);

  Ser_Child_List(f, prev, namelen, NUMBER(where) - 1, &CHILD(where, 1));
}

/****************************** GN ********************************
** GN; used internally for names of Genes.
**
** The way this works is that the nodes private field points to the AL_Gene,
** which will be in either the Genes array of either CurrentGene or one of its
** ancestor genes.  */

void GN_exe() {
  PRETVAL = PRIV(THISNODE);
  Exit_Node();
}

void GN_del(struct AL_Node * where) {
  AL_Node_free(where);
}

/* int GN_dser(struct AL_Node * where, FILE * f) {
**
** This will be handled in the main deserialisation routine so that we can
** just use the gene name and not use an explicit node-name.
**
** }  */

void GN_ser(struct AL_Node * where, FILE * f,
            struct AL_Tab * prev_tab) {
  fputs(((struct AL_Gene *)PRIV(THISNODE))->Name, f);
}

/* Our type is the type of the gene we name.
** Except we're a pointer instead of a val and a
** gene instead of a ret */

void GN_attr(struct AL_Node *where, AL_Attrib buffer) {
  COPY_ATTR(buffer, N2G(where)->Type);
  LOSE_ATTR(buffer[0], REF_ATTR | FLAV_ATTR);
  ADD_ATTR(buffer[0], PNT_ATTR | GENE_ATTR);
}

int GN_iter(struct AL_Node * where, CheckFun * check, void *private) {
  return check(where, private);
}

/* Just a dummy until I confirm the new format.  */

int GN_copy(struct AL_Node * where, struct AL_Node * was_where,
            struct AL_Node * node_pool, AL_Attrib ret_type) {
  return 1;
}

/****************************** GENE ********************************
** The node holding a Gene's definition.
*/

void GENE_ser(struct AL_Node *where,
              FILE *file,
              struct AL_Tab *prev) {
  Ser_AL_Gene(N2G(where), file, 0);
}

/* Get a gene prototype or definition from a file.
** In either case, we return 0 'cos genes aren't stored in the
** AL_Node, but rather are put in AL_Gene->Genes.  */

int GENE_dser(struct AL_Node *where, FILE *file) {
  if (!DSer_AL_Gene(file)) {
    TRACEBACK("Gene deserialisiation failure.");
  }

  return 2;
}

void GENE_del(struct AL_Node *where) {
  Free_AL_Gene(N2G(where));
  AL_Node_free(where);
}

/* Copy a gene, some parts are copied here (or rather in Copy_AL_Gene, called
** from here), others are constructed during the copying of the tree:
** 
** struct AL_Gene {
**   char *Name;			<- here
**   DECL_ATTRIB(Type);			<- here
**   struct AL_Node **tree;		<- tree (obviously !)
**   int NumVars;			<- tree
**   struct AL_VarDetail *Vars;		<- tree
**   int NumPars;			<- here
**   struct AL_VarDetail *Pars;		<- here
**   int NumGenes;			<- here
**   struct AL_Gene *Genes;		<- prototypes here, definitions in tree
**   int NumLFs;			<- tree
**   struct AL_LFDetail *LFs;		<- tree
**   struct AL_Gene *Parent;		<- here
** };  */

int GENE_copy (struct AL_Node *where,
               struct AL_Node *was_where,
               struct AL_Node *node_pool,
               AL_Attrib ret_type) {

  N2G(where) = Copy_AL_Gene(N2G(was_where),
                            node_pool,
                            ret_type);

  if (N2G(where) == NULL) {
    return 0;
  }

  return 1;
}

/* first child: body of gene, same type as gene itself but convert the
**	GENE_ATTR type to RET_TYPE.
** existing sub-genes: keep type (including params) exactly the same
**	and must not insert or delete as that would invalidate all the
**	gene-pointers in the GN-nodes.  Later we can introduce a different
**	way to get-rid of un-used sub-genes, probably tagging them as
**	redundant so they can be ommitted in later generations.
** extension at end: do not permit this as required extra sub-genes will
**	already have been added during the duplications of GN-nodes. */

int GENE_res(AL_Attrib ret_type,
             struct AL_Node * where,
             int i,
             AL_Attrib buffer) {
  if (!ACCEPTS_PARS(ret_type, N2G(where)->Type)) {
    return 0;
  }
  if (i == 0) {
    COPY_ATTR(buffer, N2G(where)->Type);
    LOSE_ATTR(buffer[0], FLAV_ATTR);
    ADD_ATTR(buffer[0], RET_ATTR);
  } else if (i <= N2G(where)->NumGenes) {
    COPY_ATTR(buffer, N2G(where)->Genes[i - 1].Type);
  } else if (i == N2G(where)->NumGenes + 1) {
    buffer[0] = END_ATTR;
  } else {
    buffer[0] = ERROR_ATTR;
  }

  return 1;
}

void GENE_attr(struct AL_Node * where, AL_Attrib buffer) {
  int i;

  COPY_ATTR(buffer, N2G(where)->Type);

  for(i = 0; i < N2G(where)->NumGenes; i++) {
    MIX_FLAGS(buffer, N2G(where)->Genes[i].Type);
  }
}

int GENE_iter(struct AL_Node * where,
              CheckFun * check,
              void *private) {
  int i;

  if (!check(where, private))
    return 0;

  if (!ITER(N2G(where)->Tree, check, private))
    return 0;

  for (i = 0; i < N2G(where)->NumGenes; ++i)
    if (!ITER(N2G(where)->Genes[i].Tree, check, private))
      return 0;

  return 1;
}
